//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#ifndef GuiRegister_h
#define GuiRegister_h

#include "GuiObject.h"


enum Claim {DONT_CLAIM,CLAIM};

//*************************************************************************************

class GuiEventTarget
{
  public: 
    virtual ~GuiEventTarget();
};

//*************************************************************************************
class GuiNullEventRegisterItem;

class GuiNullEventTarget : public GuiEventTarget
{ 
   public:
     GuiNullEventTarget();
     GuiNullEventTarget(unsigned int interval);
     virtual ~GuiNullEventTarget();
     virtual void execute()=0;

     void setInterval(unsigned int interval=0);
     void destroy();

     operator const void*() const {return item;}
     operator void*()             {return item;}
     bool operator!() const       {return item==0;}

  private:
     GuiNullEventRegisterItem* item;
     friend class GuiNullEventRegisterItem;

     GuiNullEventTarget(const GuiNullEventTarget&); //copying not allowed - not defined;
     GuiNullEventTarget& operator=(const GuiNullEventTarget&);
};


//*************************************************************************************

class GuiToolboxRegisterItem;

class GuiToolboxEventTarget : public GuiEventTarget
{ 
   public:
     GuiToolboxEventTarget();
     GuiToolboxEventTarget(const GuiObject*,int event_code);
     virtual ~GuiToolboxEventTarget();
     virtual Claim execute(GuiToolboxEvent&,const GuiIdBlock&)=0;
     void          setListener(const GuiObject*,int event_code);
     void          destroy();

     operator const void*() const {return item;}
     operator void*()             {return item;}
     bool operator!() const       {return item==0;}

  private:
     GuiToolboxRegisterItem* item;
     friend class GuiToolboxRegisterItem;

     GuiToolboxEventTarget(const GuiToolboxEventTarget&); //copying not allowed - not defined;
     GuiToolboxEventTarget& operator=(const GuiToolboxEventTarget&);
};

//*************************************************************************************

class GuiWimpRegisterItem;

class GuiWimpEventTarget : public GuiEventTarget
{ 
   public:
     GuiWimpEventTarget();
     GuiWimpEventTarget(const GuiObject*,int event_code);
     virtual ~GuiWimpEventTarget();
     virtual Claim execute(GuiWimpPollBlock&,const GuiIdBlock&)=0;
     void          setListener(const GuiObject*,int event_code);
     void          destroy();

     operator const void*() const {return item;}
     operator void*()             {return item;}
     bool operator!() const       {return item==0;}

  private:
     GuiWimpRegisterItem* item;
     friend class GuiWimpRegisterItem;

     GuiWimpEventTarget(const GuiWimpEventTarget&); //copying not allowed - not defined;
     GuiWimpEventTarget& operator=(const GuiWimpEventTarget&);
};


//*************************************************************************************

class GuiMessageRegisterItem;

class GuiMessageEventTarget : public GuiEventTarget
{ 
   public:
     GuiMessageEventTarget();
     GuiMessageEventTarget(int message_code);
     virtual ~GuiMessageEventTarget();
     virtual Claim execute(GuiWimpMessage&) = 0;
     void setListener(int message_code);
     void destroy();

     operator const void*() const {return item;}
     operator void*()             {return item;}
     bool operator!() const       {return item==0;}

  private:
     GuiMessageRegisterItem* item;
     friend class GuiMessageRegisterItem;

     GuiMessageEventTarget(const GuiMessageEventTarget&); //copying not allowed - not defined;
     GuiMessageEventTarget& operator=(const GuiMessageEventTarget&);
};

//*************************************************************************************

struct GuiRegister
{
    static void          initialise(GuiIdBlock& id_block);
    static void          poll(int *event_code=0, GuiWimpPollBlock* poll_block=0, int* poll_word=0);
    static unsigned int  get_null_event_time();
  private:
    static void          dispatch(int event_code,GuiWimpPollBlock&);
}; 

#endif
