//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#include "file.h"
#include "swis.h"


// returns file_type or len if int* 's !=0
bool isFile(const char* filename, int *filetype,int *length)
{
  DirectoryObject obj_type=fileInfo(filename,filetype,length);
  return obj_type==Dob_File;
}

//***********************************************************

DirectoryObject fileInfo(const char* filename,int *file_type,int *length)
{
  int len,load;
  DirectoryObject object_type=Dob_NotFound;
  if (_swix(OS_File,_INR(0,1) | _OUT(0) |_OUT(2) | _OUT(4),
                 5,filename,&object_type,&load,&len)==0)
     {
     if (file_type) *file_type=((load>>8) & 0xfff);
     if (length) *length=len;
     }
  return object_type;
}

//***********************************************************

bool isDirectory(const char* dirname)
{
  DirectoryObject dir_o=fileInfo(dirname);
  return dir_o==Dob_Directory || dir_o==Dob_ImageFile;
}

//***********************************************************

void setFileType(const char*filename, int type)
{
  if (type==0xdeaddead)
     {
     //set load and exec addresses to 0xdeaddead used while file is loading
     _swix(OS_File,_INR(0,2),2,filename,type);
     _swix(OS_File,_INR(0,1)|_IN(3),3,filename,type);
     }
  else
     {
     _swix(OS_File,_INR(0,2),18,filename,type);
     }
}

//***********************************************************

bool createDirectory(const char* dirname)
{
   return (_swix(OS_File,_INR(0,1) | _IN(4),8,dirname,0)==0);
}



