//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#ifndef Clip_h
#define Clip_h

#include "GuiBBox.h"
#include <limits.h>

class ClipBox : public GuiBBox
{ 
  public:
    ClipBox*      next;
    int*          path;
    int           removePath;
};

// clip units are internal draw units
class Clip
{
  public:
    Clip();
    ~Clip();
    int          init(const GuiBBox&); // returns isOk()
    int          isOk() const   {return box!=0;}
    void push();
    void pop();
    int clip(int& x,int& y) const
                     {
                       int mask=0;
                       if (box)
                       {
                         if  (x<box->xmin) {x=box->xmin; mask|=1;}
                         else if (x>box->xmax) {x=box->xmax;mask|=2;}

                         if  (y<box->ymin) {y=box->ymin;mask|=4;}
                         else if (y>box->ymax) {y=box->ymax;mask|=8;}
                       }
                       return mask;
                     }
    
    const int*     getPath() const     {return box?box->path:0;}
    void           setPath(int* pathArray);
    const GuiBBox* getBBox()           {return box;}
  private:
    ClipBox*       box;

    void           removePath();
    void           clear();
    void           doPop();
};

inline void Boundary_start(GuiBBox& box)
{
  box(INT_MAX,INT_MAX,INT_MIN,INT_MIN);
}

//*******************************************************************

inline void Boundary_point(GuiBBox& box,int x,int y)
{
  if (x < box.xmin) box.xmin=x;
  if (x > box.xmax) box.xmax=x;
  if (y < box.ymin) box.ymin=y;
  if (y > box.ymax) box.ymax=y;
}

//*******************************************************************

#endif
