//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#ifndef DocViewToolbar_h
#define DocViewToolbar_h

#include "GuiWindow.h"
#include "GuiGadget.h"
#include "PageHistory.h"
#include "Bookmarks.h"
#include "NumberField.h"

class DocView;

class DocViewToolbar : public GuiWindow
{
  public:
    DocViewToolbar(DocView&, GuiWindow& ancestor);
    ~DocViewToolbar();

    int          getPage()                        {return currentPage;}
    int          getLogicalPageOffset()           {return logicalPageOffset;}
    void         flushPendingHistory()            {history.addPending();}
    void         setFirstPage(int page,int logicalPageOffset,int page_count, bool add_to_history=1);
    void         goBack();
    void         openWindow(GuiWindow& parent,GuiGetWindowStateBlock& parent_state);
    void         setPage(int new_page, bool add_to_history=1);
    void         setFocus()                       {page.setFocus();}
  private:
    DocView&         view;
    NumberField      page;
    NumberDisplay    lastPage;
    GuiSlider        pageSlider;
    GuiToolAction    outline;

    int              logicalPageOffset;
    int              currentPage;
    int              currentStep;
    int              pageCount;
    int              oldWidth;
    PageHistory      history;
    Bookmarks        bookmarks;


    GUI_DECLARE_EVENT_TARGETS(DocViewToolbar);

    GuiToolboxTarget showOutlineTarget;
    Claim showOutline(GuiToolboxEvent&,const GuiIdBlock&);

    GuiToolboxTarget gViewPostscriptFileTarget;
    Claim gViewPostscriptFile(GuiToolboxEvent&,const GuiIdBlock&);

    GuiToolboxTarget gViewPdfFileTarget;
    Claim gViewPdfFile(GuiToolboxEvent&,const GuiIdBlock&);

    GuiToolboxTarget adjusterClickedTarget;
    Claim adjusterClicked(GuiToolboxEvent&,const GuiIdBlock&);

    GuiToolboxTarget readPageTarget;
    Claim readPage(GuiToolboxEvent&,const GuiIdBlock&);

    GuiToolboxTarget sliderValueChangedTarget;
    Claim sliderValueChanged(GuiToolboxEvent&,const GuiIdBlock&);

    GuiToolboxTarget nextPartPageTarget;
    GuiToolboxTarget nextPageTarget;
    GuiToolboxTarget fastNextPageTarget;
    Claim nextPage(GuiToolboxEvent&,const GuiIdBlock&);

    GuiToolboxTarget previousPartPageTarget;
    GuiToolboxTarget previousPageTarget;
    GuiToolboxTarget fastPreviousPageTarget;
    Claim previousPage(GuiToolboxEvent&,const GuiIdBlock&);

    GuiToolboxTarget logicalPageNumberATBSTarget;
    Claim logicalPageNumberATBS(GuiToolboxEvent&,const GuiIdBlock&);

    GuiToolboxTarget logicalPageNumberSetTarget;
    Claim logicalPageNumberSet(GuiToolboxEvent&,const GuiIdBlock&);
};
#endif
