//--------------------------------------------------------------------------
//
//   Copyright (c) 2002, Colin Granville
//
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or
//   without modification, are permitted provided that the following 
//   conditions are met:
//
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
//
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution.
//
//      * The name Colin Granville may not be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
//   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
//   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//   STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
//   OF THE POSSIBILITY OF SUCH DAMAGE.
//
//--------------------------------------------------------------------------

#ifndef Document_h
#define Document_h

#include <string>
#include "GuiTargets.h"
#include "Catalog.h"
#include "swis.h"
#include "OutputDev.h"
#include <stdio.h>
#include "Node.h"
#include "PDFDoc.h"
#include "FileInfo.h"

using namespace std;

class View;
class Links;
class Outline;
class DocView;

class EraseScrapFile
{
  public:
     EraseScrapFile(const char* f) : filename(f) {}
     ~EraseScrapFile();

  private:
    string filename;
};

class Document : public Node
{
  public:
    // document will delete itself
    // returns *mess=0 if it is a PDF file
    friend Document* makeDocument(Node& application,const char* file_name);
    virtual ~Document();
    virtual bool  hasChildNodes();
    PDFDoc&       getPDFDoc()                       {return *doc;}
    int           getPageCount()                    {return getPDFDoc().getNumPages();}
    Catalog*      getCatalog()                      {return getPDFDoc().getCatalog();}
    void          getPage(int num,OutputDev*,int dpi,int rotate);
    Links*        makeLinks(int page_num);          //caller deletes links
    const string& getFileName()                     {return fileName;}
    int           isOk()                            {return doc && getPDFDoc().isOk();}

/*    int           findPage(Ref& ref)                {return getPDFDoc().findPage(ref.num,ref.gen);} */
    int           findPage(Ref ref)                {return getPDFDoc().findPage(ref.num,ref.gen);}
    LinkDest*     findDest(GString *name)           {return getPDFDoc().findDest(name);}

    virtual Node* removeChild(Node*);
    virtual Node* addChild(Node*);
    
    Outline*      getOutline()                      {return outline;}

    const FileInfoData& getFileInfoData()           {return fileInfoData;}

    DECLARE_RTTI();
    
  private:
    Document(Node& application,const char* file_name); //create with make
    string               fileName;
    EraseScrapFile       eraseScrapFile;
    PDFDoc*              doc;
    Outline*             outline;
    void                 setViewTitles();
    string               getInfo(const char* key);
    FileInfoData         fileInfoData;
};

#endif
